@@ -1,9 +1,10 @@
 import subprocess
 import shlex
-from .command import SiteCommand, CmdlineError
+from .command import SiteCommand, Fail
+from staticsite.page_filter import PageFilter
 import logging
 
-log = logging.getLogger()
+log = logging.getLogger("edit")
 
 
 class Edit(SiteCommand):
@@ -18,7 +19,7 @@ class Edit(SiteCommand):
                 if m[1:] not in tags:
                     return False
             elif (m.lower() not in page.meta.get("title", "").lower()
-                  and m.lower() not in page.src_relpath.lower()):
+                  and m.lower() not in page.src.relpath.lower()):
                 return False
         return True
 
@@ -49,7 +50,7 @@ class Edit(SiteCommand):
                 print("{idx}: {date} {relpath} {title}".format(
                     idx=idx,
                     date=page.meta["date"].strftime("%Y-%m-%d"),
-                    relpath=page.src_relpath,
+                    relpath=page.src.relpath,
                     title=page.meta.get("title", "{no title}")))
             print()
 
@@ -59,7 +60,7 @@ class Edit(SiteCommand):
             elif pagination and selection == "p":
                 first = max(0, first - self.MENU_SIZE)
                 continue
-            elif pagination and selection == "n":
+            elif pagination and selection == "n" or selection == "":
                 first = min(first + self.MENU_SIZE, self.MENU_SIZE * (len(pages) // self.MENU_SIZE))
                 continue
 
@@ -73,9 +74,9 @@ class Edit(SiteCommand):
     def run(self):
         site = self.load_site()
 
-        # Build a list of all findable pages sorted with the newest first
-        pages = [p for p in site.pages.values() if p.FINDABLE]
-        pages.sort(key=lambda x: x.meta["date"], reverse=True)
+        # Build a list of all findable pages, present on disk, sorted with the newest first
+        f = PageFilter(site, sort="-date")
+        pages = [page for page in f.filter(site.pages.values()) if page.src.stat is not None]
 
         selected = []
         for page in pages:
@@ -84,7 +85,7 @@ class Edit(SiteCommand):
             selected.append(page)
 
         if len(selected) == 0:
-            raise CmdlineError("No page found matching {}".format(" ".join(shlex.quote(x) for x in self.args.match)))
+            raise Fail("No page found matching {}".format(" ".join(shlex.quote(x) for x in self.args.match)))
 
         if len(selected) == 1:
             page = selected[0]
