@@ -1,18 +1,18 @@
-from staticsite.render import RenderedString
-from staticsite import Page, Feature
-from staticsite.utils import parse_front_matter, write_front_matter
+from __future__ import annotations
+from typing import List, Tuple
+from staticsite import Page, Feature, File
+from staticsite.utils import front_matter
 from staticsite.archetypes import Archetype
+from staticsite.contents import ContentDir, Dir
+from staticsite.utils import lazy
+from staticsite.utils.typing import Meta
 import jinja2
 import os
 import io
-import pytz
-import datetime
 import markdown
-import dateutil.parser
-from urllib.parse import urlparse, urlunparse
 import logging
 
-log = logging.getLogger()
+log = logging.getLogger("markdown")
 
 
 class LinkResolver(markdown.treeprocessors.Treeprocessor):
@@ -42,9 +42,11 @@ class LinkResolver(markdown.treeprocessors.Treeprocessor):
 
         Returns None if the URL does not need changing, else returns the new URL.
         """
+        new_url = self.substituted.get(url)
+        if new_url is not None:
+            return new_url
+
         from markdown.util import AMP_SUBSTITUTE
-        if not url:
-            return None
         if url.startswith(AMP_SUBSTITUTE):
             # Possibly an overencoded mailto: link.
             # see https://bugs.debian.org/816218
@@ -52,28 +54,13 @@ class LinkResolver(markdown.treeprocessors.Treeprocessor):
             # Markdown then further escapes & with utils.AMP_SUBSTITUTE, so
             # we look for it here.
             return None
-        parsed = urlparse(url)
-        if parsed.scheme or parsed.netloc:
-            return None
-        if not parsed.path:
-            return None
-        dest = self.page.resolve_link(parsed.path)
-        # Also allow .md extension in
-        if dest is None and parsed.path.endswith(".md"):
-            dirname, basename = os.path.split(parsed.path)
-            if basename in ("index.md", "README.md"):
-                dest = self.page.resolve_link(dirname)
-            else:
-                dest = self.page.resolve_link(parsed.path[:-3])
-        if dest is None:
-            log.warn("%s: internal link %r does not resolve to any site page", self.page.src.relpath, url)
+
+        new_url = self.page.resolve_url(url)
+        if new_url is url:
             return None
 
-        res = urlunparse(
-            (parsed.scheme, parsed.netloc, dest.dst_link, parsed.params, parsed.query, parsed.fragment)
-        )
-        self.substituted[url] = res
-        return res
+        self.substituted[url] = new_url
+        return new_url
 
 
 class StaticSiteExtension(markdown.extensions.Extension):
@@ -94,10 +81,8 @@ class MarkdownPages(Feature):
     """
     Render ``.md`` markdown pages, with front matter.
 
-    See doc/markdown.md for details.
+    See doc/reference/markdown.md for details.
     """
-    RUN_BEFORE = ["tags"]
-
     def __init__(self, *args, **kw):
         super().__init__(*args, **kw)
         md_staticsite = StaticSiteExtension()
@@ -109,9 +94,6 @@ class MarkdownPages(Feature):
             output_format="html5",
         )
         self.link_resolver = md_staticsite.link_resolver
-        # Cached templates
-        self._page_template = None
-        self._redirect_template = None
 
         self.j2_filters["markdown"] = self.jinja2_markdown
 
@@ -121,29 +103,14 @@ class MarkdownPages(Feature):
     def jinja2_markdown(self, context, mdtext):
         return jinja2.Markup(self.render_snippet(context.parent["page"], mdtext))
 
-    @property
-    def page_template(self):
-        if not self._page_template:
-            self._page_template = self.site.theme.jinja2.get_template("page.html")
-        return self._page_template
-
-    @property
-    def redirect_template(self):
-        if not self._redirect_template:
-            self._redirect_template = self.site.theme.jinja2.get_template("redirect.html")
-        return self._redirect_template
-
     def render_page(self, page):
         """
         Render markdown in the context of the given page.
-
-        It renders the page content by default, unless `content` is set to a
-        different markdown string.
         """
         self.link_resolver.set_page(page)
 
         cached = self.render_cache.get(page.src.relpath)
-        if cached and cached["mtime"] != page.mtime:
+        if cached and cached["mtime"] != page.src.stat.st_mtime:
             cached = None
         if cached:
             for src, dest in cached["paths"]:
@@ -159,7 +126,7 @@ class MarkdownPages(Feature):
         rendered = self.markdown.convert(content)
 
         self.render_cache.put(page.src.relpath, {
-            "mtime": page.mtime,
+            "mtime": page.src.stat.st_mtime,
             "rendered": rendered,
             "paths": list(self.link_resolver.substituted.items()),
         })
@@ -179,15 +146,85 @@ class MarkdownPages(Feature):
         self.markdown.reset()
         return self.markdown.convert(content)
 
-    def try_load_page(self, src):
-        if not src.relpath.endswith(".md"):
-            return None
-        return MarkdownPage(self, src)
+    def load_dir(self, sitedir: ContentDir) -> List[Page]:
+        taken: List[str] = []
+        pages: List[Page] = []
+        for fname, src in sitedir.files.items():
+            if not fname.endswith(".md"):
+                continue
+            taken.append(fname)
+
+            meta = sitedir.meta_file(fname)
+            if fname not in ("index.md", "README.md"):
+                meta["site_path"] = os.path.join(meta["site_path"], fname[:-3])
+
+            try:
+                fm_meta, body = self.load_file_meta(sitedir, src, fname)
+            except Exception as e:
+                log.warn("%s: Failed to parse markdown page front matter (%s): skipped", src, e)
+                log.debug("%s: Failed to parse markdown page front matter: skipped", src, exc_info=e)
+                continue
+
+            meta.update(fm_meta)
+
+            page = MarkdownPage(self, src, meta=meta, dir=sitedir, body=body)
+            pages.append(page)
+
+        for fname in taken:
+            del sitedir.files[fname]
+
+        return pages
+
+    def load_dir_meta(self, sitedir: ContentDir):
+        # Load front matter from index.md
+        # Do not try to load front matter from README.md, as one wouldn't
+        # clutter a repo README with staticsite front matter
+        index = sitedir.files.get("index.md")
+        if index is None:
+            return
+
+        try:
+            with open(index.abspath, "rb") as fd:
+                fmt, meta = front_matter.read_partial(fd)
+        except Exception as e:
+            log.debug("%s: failed to parse front matter", index.relpath, exc_info=e)
+            log.warn("%s: failed to parse front matter", index.relpath)
+        else:
+            sitedir.add_dir_config(meta)
+
+    def load_file_meta(self, sitedir, src, fname) -> Tuple[Meta, List[str]]:
+        """
+        Load metadata for a file.
+
+        Returns the metadata and the markdown lines for the rest of the file
+        """
+        # Read the contents
+
+        # Parse separating front matter and markdown content
+        with sitedir.open(fname, src, "rb") as fd:
+            fmt, meta = front_matter.read_partial(fd)
+
+            body = []
+            for line in fd:
+                body.append(line.strip().decode())
+
+            # Remove leading empty lines
+            while body and not body[0]:
+                body.pop(0)
+
+            # Read title from first # title if not specified in metadata
+            if not meta.get("title", ""):
+                if body and body[0].startswith("# "):
+                    meta["title"] = body.pop(0)[2:].strip()
+
+                # Remove leading empty lines again
+                while body and not body[0]:
+                    body.pop(0)
+
+        return meta, body
 
     def try_load_archetype(self, archetypes, relpath, name):
-        if not relpath.endswith(".md"):
-            return None
-        if not (relpath.endswith(name) or relpath.endswith(name + ".md")):
+        if os.path.basename(relpath) != name + ".md":
             return None
         return MarkdownArchetype(archetypes, relpath, self)
 
@@ -199,7 +236,7 @@ def parse_markdown_with_front_matter(fd):
     Returns two arrays: one with the lines of front matter and one with the
     lines of markdown.
     """
-    front_matter = []
+    fmatter = []
     body = []
 
     front_matter_end = None
@@ -218,13 +255,13 @@ def parse_markdown_with_front_matter(fd):
                 in_front_matter = False
 
         if in_front_matter:
-            front_matter.append(line)
+            fmatter.append(line)
             if lineno > 1 and line == front_matter_end:
                 in_front_matter = False
         else:
             body.append(line)
 
-    return front_matter, body
+    return fmatter, body
 
 
 class MarkdownArchetype(Archetype):
@@ -238,11 +275,12 @@ class MarkdownArchetype(Archetype):
         # Reparse the rendered version
         with io.StringIO(rendered) as fd:
             # Reparse it separating front matter and markdown content
-            front_matter, body = parse_markdown_with_front_matter(fd)
+            fmatter, body = parse_markdown_with_front_matter(fd)
         try:
-            style, meta = parse_front_matter(front_matter)
-        except Exception:
-            log.exception("archetype %s: failed to parse front matter", self.relpath)
+            style, meta = front_matter.parse(fmatter)
+        except Exception as e:
+            log.debug("archetype %s: failed to parse front matter", self.relpath, exc_info=e)
+            log.warn("archetype %s: failed to parse front matter", self.relpath)
 
         # Make a copy of the full parsed metadata
         archetype_meta = dict(meta)
@@ -251,9 +289,9 @@ class MarkdownArchetype(Archetype):
         meta.pop("path", None)
 
         # Reserialize the page with the edited metadata
-        front_matter = write_front_matter(meta, style)
+        fmatter = front_matter.write(meta, style)
         with io.StringIO() as fd:
-            fd.write(front_matter)
+            fd.write(fmatter)
             print(file=fd)
             for line in body:
                 print(line, file=fd)
@@ -261,78 +299,24 @@ class MarkdownArchetype(Archetype):
 
         return archetype_meta, post_body
 
-#    def read_md(self, **kw):
-#        """
-#        Process the archetype returning its parsed front matter in a dict, and
-#        its contents in a string
-#        """
-#
-#        return style, meta, body
-
 
 class MarkdownPage(Page):
     TYPE = "markdown"
 
-    FINDABLE = True
+    def __init__(
+            self, mdpages: MarkdownPages, src: File, meta: Meta, dir: Dir, body: List[str]):
+        super().__init__(site=mdpages.site, src=src, meta=meta, dir=dir)
 
-    def __init__(self, mdpages, src):
-        dirname, basename = os.path.split(src.relpath)
-        if basename in ("index.md", "README.md"):
-            linkpath = dirname
-        else:
-            linkpath = os.path.splitext(src.relpath)[0]
-        super().__init__(
-            site=mdpages.site,
-            src=src,
-            src_linkpath=linkpath,
-            dst_relpath=os.path.join(linkpath, "index.html"),
-            dst_link=os.path.join(mdpages.site.settings.SITE_ROOT, linkpath))
+        self.meta["build_path"] = os.path.join(meta["site_path"], "index.html")
+
+        # Indexed by default
+        self.meta.setdefault("indexed", True)
 
         # Shared markdown environment
         self.mdpages = mdpages
 
-        # Sequence of lines found in the front matter
-        self.front_matter = []
-
         # Sequence of lines found in the body
-        self.body = []
-
-        # Markdown content of the page rendered into html
-        self.md_html = None
-
-        # Modification time of the file
-        self.mtime = self.src.stat.st_mtime
-
-        # Read the contents
-        if self.meta.get("date", None) is None:
-            self.meta["date"] = pytz.utc.localize(datetime.datetime.utcfromtimestamp(self.mtime))
-
-        # Parse separating front matter and markdown content
-        with open(self.src.abspath, "rt") as fd:
-            self.front_matter, self.body = parse_markdown_with_front_matter(fd)
-
-        try:
-            style, meta = parse_front_matter(self.front_matter)
-            self.meta.update(**meta)
-        except Exception:
-            log.exception("%s: failed to parse front matter", self.src.relpath)
-
-        # Remove leading empty lines
-        while self.body and not self.body[0]:
-            self.body.pop(0)
-
-        # Read title from first # title if not specified in metadata
-        if not self.meta.get("title", ""):
-            if self.body and self.body[0].startswith("# "):
-                self.meta["title"] = self.body.pop(0)[2:].strip()
-
-                # Remove leading empty lines again
-                while self.body and not self.body[0]:
-                    self.body.pop(0)
-
-        date = self.meta.get("date", None)
-        if date is not None and not isinstance(date, datetime.datetime):
-            self.meta["date"] = dateutil.parser.parse(date)
+        self.body = body
 
     def get_content(self):
         return "\n".join(self.body)
@@ -340,35 +324,9 @@ class MarkdownPage(Page):
     def check(self, checker):
         self.mdpages.render_page(self)
 
-    @property
+    @lazy
     def content(self):
-        if self.md_html is None:
-            self.md_html = self.mdpages.render_page(self)
-        return self.md_html
-
-    def render(self):
-        res = {}
-
-        html = self.mdpages.page_template.render(
-            page=self,
-            content=self.content,
-            **self.meta
-        )
-        res[self.dst_relpath] = RenderedString(html)
-
-        for relpath in self.meta.get("aliases", ()):
-            html = self.mdpages.redirect_template.render(
-                page=self,
-            )
-            res[os.path.join(relpath, "index.html")] = RenderedString(html)
-
-        return res
-
-    def target_relpaths(self):
-        res = [self.dst_relpath]
-        for relpath in self.meta.get("aliases", ()):
-            res.append(os.path.join(relpath, "index.html"))
-        return res
+        return self.mdpages.render_page(self)
 
 
 FEATURES = {
