@@ -1,5 +1,5 @@
 from unittest import TestCase
-from staticsite.site import Site
+from . import utils as test_utils
 from staticsite.asset import Asset
 from staticsite.file import File
 from contextlib import contextmanager
@@ -34,16 +34,15 @@ def override_tz(val):
 
 
 class TestAsset(TestCase):
+    @test_utils.assert_no_logs()
     def test_timestamps(self):
-        site = Site()
-        site.settings.THEME = os.path.join(os.getcwd(), "example", "theme")
-        site.load()
+        site = test_utils.Site()
 
         with override_tz("Pacific/Samoa"):
             with tempfile.NamedTemporaryFile() as f:
                 # $ TZ=UTC date +%s --date="2016-11-01" → 1477958400
                 os.utime(f.name, (1477958400, 1477958400))
-                src = File(os.path.basename(f.name), root=os.path.dirname(f.name), abspath=f.name, stat=os.stat(f.name))
-                page = Asset(site, src)
+                src = File(os.path.basename(f.name), abspath=f.name, stat=os.stat(f.name))
+                page = Asset(site, src, meta={"site_path": src.relpath})
 
                 self.assertEqual(page.meta["date"], datetime.datetime(2016, 11, 1, 0, 0, 0, tzinfo=pytz.utc))
